﻿
namespace RedPoint.ReefStatus.Common.UI.Controls
{
    using System.Windows;

    /// <summary>
    /// Custom button
    /// </summary>
    public class ButtonChrome : DependencyObject
    {
        /// <summary>
        /// Corner Radius Property
        /// </summary>
        public static readonly DependencyProperty CornerRadiusProperty =
            DependencyProperty.RegisterAttached("CornerRadius", typeof(CornerRadius), typeof(ButtonChrome), new FrameworkPropertyMetadata(new CornerRadius(0), FrameworkPropertyMetadataOptions.AffectsRender));

        /// <summary>
        /// ShowCircle Property
        /// </summary>
        public static readonly DependencyProperty ShowCircleProperty =
           DependencyProperty.RegisterAttached("ShowCircle", typeof(bool), typeof(ButtonChrome), new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.AffectsRender));

        /// <summary>
        /// Sets the show circle.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <param name="value">if set to <c>true</c> [value].</param>
        public static void SetShowCircle(UIElement element, bool value)
        {
            element.SetValue(ShowCircleProperty, value);
        }

        /// <summary>
        /// Gets the show circle.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <returns>If the Circle is shown</returns>
        public static bool GetShowCircle(UIElement element)
        {
            return (bool)element.GetValue(ShowCircleProperty);
        }

        /// <summary>
        /// Sets the corner radius.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <param name="value">The value.</param>
        public static void SetCornerRadius(UIElement element, CornerRadius value)
        {
            element.SetValue(CornerRadiusProperty, value);
        }

        /// <summary>
        /// Gets the corner radius.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <returns>The corner radius</returns>
        public static CornerRadius GetCornerRadius(UIElement element)
        {
            return (CornerRadius)element.GetValue(CornerRadiusProperty);
        }
    }
}
