﻿
namespace RedPoint.ReefStatus.Common.UI.Controls
{
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;

    /// <summary>
    /// Crome hover and selected state
    /// </summary>
    public class Chrome : UserControl
    {
        /// <summary>
        /// IsPressed Property
        /// </summary>
        public static readonly DependencyProperty IsPressedProperty =
            DependencyProperty.Register("IsPressed", typeof(bool), typeof(Chrome));

        /// <summary>
        /// Corner Radius Property
        /// </summary>
        public static readonly DependencyProperty CornerRadiusProperty =
            DependencyProperty.Register("CornerRadius", typeof(CornerRadius), typeof(Chrome));

        /// <summary>
        /// Is Checked Property
        /// </summary>
        public static readonly DependencyProperty IsCheckedProperty =
            DependencyProperty.Register("IsChecked", typeof(bool), typeof(Chrome));

         /// <summary>
        /// IsHighlighted Property
        /// </summary>
        public static readonly DependencyProperty IsHighlightedProperty =
            DependencyProperty.Register("IsHighlighted", typeof(bool), typeof(Chrome));

        /// <summary>
        /// PressedBorderBrush Property
        /// </summary>
        public static readonly DependencyProperty PressedBorderProperty =
            DependencyProperty.Register("PressedBorder", typeof(Brush), typeof(Chrome));

        /// <summary>
        /// PressedBorderBrush Property
        /// </summary>
        public static readonly DependencyProperty HoverBorderProperty =
            DependencyProperty.Register("HoverBorder", typeof(Brush), typeof(Chrome));

        /// <summary>
        /// PressedBorderBrush Property
        /// </summary>
        public static readonly DependencyProperty PressedBackgroundProperty =
            DependencyProperty.Register("PressedBackground", typeof(Brush), typeof(Chrome));

        /// <summary>
        /// PressedBorderBrush Property
        /// </summary>
        public static readonly DependencyProperty HoverBackgroundProperty =
            DependencyProperty.Register("HoverBackground", typeof(Brush), typeof(Chrome));

        /// <summary>
        /// ShowCircle Property
        /// </summary>
        public static readonly DependencyProperty ShowCircleProperty =
           DependencyProperty.Register("ShowCircle", typeof(bool), typeof(Chrome));

        /// <summary>
        /// Gets or sets a value indicating whether [show circle].
        /// </summary>
        /// <value><c>true</c> if [show circle]; otherwise, <c>false</c>.</value>
        public bool ShowCircle
        {
            get { return (bool)GetValue(ShowCircleProperty); }
            set { SetValue(ShowCircleProperty, value); }
        }

        /// <summary>
        /// Gets or sets the pressed border brush.
        /// </summary>
        /// <value>The pressed border brush.</value>
        public Brush PressedBorder
        {
            get { return (Brush)GetValue(PressedBorderProperty); }
            set { SetValue(PressedBorderProperty, value); }
        }

        /// <summary>
        /// Gets or sets the hover border brush.
        /// </summary>
        /// <value>The hover border brush.</value>
        public Brush HoverBorder
        {
            get { return (Brush)GetValue(HoverBorderProperty); }
            set { SetValue(HoverBorderProperty, value); }
        }

        /// <summary>
        /// Gets or sets the pressed background brush.
        /// </summary>
        /// <value>The pressed background brush.</value>
        public Brush PressedBackground
        {
            get { return (Brush)GetValue(PressedBackgroundProperty); }
            set { SetValue(PressedBackgroundProperty, value); }
        }

        /// <summary>
        /// Gets or sets the hover background.
        /// </summary>
        /// <value>The hover background.</value>
        public Brush HoverBackground
        {
            get { return (Brush)GetValue(HoverBackgroundProperty); }
            set { SetValue(HoverBackgroundProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is pressed.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is pressed; otherwise, <c>false</c>.
        /// </value>
        public bool IsHighlighted
        {
            get { return (bool)GetValue(IsHighlightedProperty); }
            set { SetValue(IsHighlightedProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is pressed.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is pressed; otherwise, <c>false</c>.
        /// </value>
        public bool IsPressed
        {
            get { return (bool)GetValue(IsPressedProperty); }
            set { SetValue(IsPressedProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is checked.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is checked; otherwise, <c>false</c>.
        /// </value>
        public bool IsChecked
        {
            get { return (bool)GetValue(IsCheckedProperty); }
            set { SetValue(IsCheckedProperty, value); }
        }

        /// <summary>
        /// Gets or sets the corner radius.
        /// </summary>
        /// <value>The corner radius.</value>
        public CornerRadius CornerRadius
        {
            get { return (CornerRadius)GetValue(CornerRadiusProperty); }
            set { SetValue(CornerRadiusProperty, value); }
        }
    }
}
