﻿
namespace RedPoint.ReefStatus.Common.UI.Controls
{
    using System.Windows;
    using System.Windows.Controls;

    /// <summary>
    /// Arrow Direction
    /// </summary>
    public enum ArrowDirection
    {
        /// <summary>
        /// Show up Arrow
        /// </summary>
        Up,

        /// <summary>
        /// Show down Arrow
        /// </summary>
        Down,

        /// <summary>
        /// Show left Arrow
        /// </summary>
        Left,

        /// <summary>
        /// Show Right Arrow
        /// </summary>
        Right,
    }

    /// <summary>
    /// Arrow Control
    /// </summary>
    public class Arrow : UserControl
    {
        /// <summary>
        /// IsHighlighted Property
        /// </summary>
        public static readonly DependencyProperty IsHighlightedProperty = DependencyProperty.Register(
            "IsHighlighted", 
            typeof(bool), 
            typeof(Arrow), 
            new FrameworkPropertyMetadata(false, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault | FrameworkPropertyMetadataOptions.AffectsRender));

        /// <summary>
        /// Direction Property
        /// </summary>
        public static readonly DependencyProperty DirectionProperty = DependencyProperty.Register(
            "Direction",
            typeof(ArrowDirection),
            typeof(Arrow),
            new FrameworkPropertyMetadata(ArrowDirection.Up, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault | FrameworkPropertyMetadataOptions.AffectsRender));

        /// <summary>
        /// Gets or sets a value indicating whether this instance is pressed.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is pressed; otherwise, <c>false</c>.
        /// </value>
        public bool IsHighlighted
        {
            get { return (bool)GetValue(IsHighlightedProperty); }
            set { SetValue(IsHighlightedProperty, value); }
        }

        /// <summary>
        /// Gets or sets the direction.
        /// </summary>
        /// <value>The direction.</value>
        public ArrowDirection Direction
        {
            get { return (ArrowDirection)GetValue(DirectionProperty); }
            set { SetValue(DirectionProperty, value); }
        }
    }
}
