﻿// <copyright file="DatabaseConnection.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common.Database
{
    using System;
    using System.IO;

    /// <summary>
    /// Database Connection
    /// </summary>
    public class DatabaseConnection
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DatabaseConnection"/> class.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="type">The type.</param>
        public DatabaseConnection(string connection, DatabaseType type)
        {
            this.DatabaseType = type;
            this.Connection = connection;
        }

        /// <summary>
        /// Gets the connection.
        /// </summary>
        /// <value>The connection.</value>
        public string Connection { get; private set; }

        /// <summary>
        /// Gets the type of the database.
        /// </summary>
        /// <value>The type of the database.</value>
        public DatabaseType DatabaseType { get; private set; }

        /// <summary>
        /// Creates the specified connection.
        /// </summary>
        /// <returns>The database Connection for the settings</returns>
        public IDataAccess Create()
        {
#if !Mono
            try
            {
                switch (this.DatabaseType)
                {
                    case DatabaseType.Sql:
                        return new SqlDataAccess(this.Connection);
                    case DatabaseType.Sqlite:
                        return new SqliteDataAccess(this.Connection);
                    case DatabaseType.Memory:
                        return new MemoryDataAccess();
                    case DatabaseType.Odbc:
                        return new OdbcDataAccess(this.Connection);
                    case DatabaseType.Ole:
                        return new OleDataAccess(this.Connection);
                    default:
                        throw new DataAccessException(201, "Unknown Database source");
                }
            }
            catch (DirectoryNotFoundException ex)
            {
                throw new DataAccessException(201, "Unable to Connect to Database", ex);
            }
#else
			
			return null; 
#endif
        }

        internal void Backup(string archiveLocation)
        {
            /*using (IDataAccess dataAccess = this.Create())
            {
                if (dataAccess != null)
                {
                    dataAccess.Backup(Connection, archiveLocation);
                }
            }*/
        }
    }
}
