﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Controler.cs" company="RedpointGames">
//   2010
// </copyright>
// <summary>
//   Controler Class
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Common.ProfiLux
{
    using System;
    using System.Collections.ObjectModel;
    using System.Linq;
#if PocketPC != true && Mono != true
    using System.Windows.Data;
    using System.Windows.Input;
#endif	
#if PocketPC != true
    using RedPoint.ReefStatus.Common.Commands;
#endif
    using RedPoint.ReefStatus.Common.Communication;
#if PocketPC != true
    using RedPoint.ReefStatus.Common.Core;
#endif
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    /// <summary>
    /// Controler Class
    /// </summary>
    public class Controler : ViewModelBase
    {
        /// <summary>
        /// Id of the controler
        /// </summary>
        private int id;

        /// <summary>
        /// Name of the controler
        /// </summary>
        private string name;

#if PocketPC != true && Mono != true
        /// <summary>
        /// Manual Illumination Command
        /// </summary>
        private ICommand manualIlluminationCommand;

        /// <summary>
        /// mainteance Command
        /// </summary>
        private ICommand maintenanceCommand;

        /// <summary>
        /// Manual Command
        /// </summary>
        private ICommand manualCommand;

        /// <summary>
        /// Feed Command
        /// </summary>
        private ICommand feedCommand;
#endif

        /// <summary>
        /// The command thread
        /// </summary>
        private CommandThread commands;

        /// <summary>
        /// Initializes a new instance of the <see cref="Controler"/> class.
        /// </summary>
        public Controler()
        {
            this.Connection = new ConnectionSettings();
            this.Info = new Info();
            this.Items = new SafeObservableCollection<BaseInfo>();
            this.Probes = new ListCollectionView(this.Items) { Filter = obj => obj is Probe };
            this.LevelSensors = new ListCollectionView(this.Items) { Filter = obj => obj is LevelSensor };
            this.SPorts = new ListCollectionView(this.Items) { Filter = obj => obj is SPort };
            this.LPorts = new ListCollectionView(this.Items) { Filter = obj => obj is LPort };
            this.UserValues = new ListCollectionView(this.Items) { Filter = obj => obj is UserInfo };
            this.DigitalInputs = new ListCollectionView(this.Items) { Filter = obj => obj is DigitalInput };
            this.Lights = new ListCollectionView(this.Items) { Filter = obj => obj is Light };
            this.DosingPumps = new ListCollectionView(this.Items) { Filter = obj => obj is DosingPump };
            this.CurrentPumps = new ListCollectionView(this.Items) { Filter = obj => obj is CurrentPump };
        }

        /// <summary>
        /// Gets or sets the connection.
        /// </summary>
        /// <value>The connection.</value>
        public ConnectionSettings Connection { get; set; }

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id.</value>
        public int Id
        {
            get
            {
                return this.id;
            }

            set
            {
                this.id = value;
                if (string.IsNullOrEmpty(this.Name))
                {
                    this.Name = "Controler" + this.id;
                }       
            }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                if (this.name != value)
                {
                    this.name = value;
                    this.OnPropertyChanged("Name");
                }
            }
        }

        /// <summary>
        /// Gets or sets the items.
        /// </summary>
        /// <value>The items.</value>
        public SafeObservableCollection<BaseInfo> Items { get; set; }

        /// <summary>
        /// Gets or sets the controler info.
        /// </summary>
        /// <value>The info.</value>
        public Info Info { get; set; }

#if PocketPC != true && Mono != true

        /// <summary>
        /// Gets the manual illumination command.
        /// </summary>
        /// <value>The manual illumination command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand ManualIlluminationCommand
        {
            get
            {
                return this.manualIlluminationCommand ??
                       (this.manualIlluminationCommand = new DelegateCommand(this.ManualIllumination));
            }
        }

        /// <summary>
        /// Gets the maintenance command.
        /// </summary>
        /// <value>The maintenance command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand MaintenanceCommand
        {
            get
            {
                return this.maintenanceCommand ?? (this.maintenanceCommand = new DelegateCommand(this.Maintenance));
            }
        }

        /// <summary>
        /// Gets the manual command.
        /// </summary>
        /// <value>The manual command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand ManualCommand
        {
            get
            {
                return this.manualCommand ?? (this.manualCommand = new DelegateCommand(this.ManualSockets));
            }
        }

        /// <summary>
        /// Gets the feed command.
        /// </summary>
        /// <value>The feed command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand FeedCommand
        {
            get
            {
                return this.feedCommand ?? (this.feedCommand = new DelegateCommand(this.Feed));
            }
        }
#endif

        /// <summary>
        /// Gets the commands.
        /// </summary>
        /// <value>The commands.</value>
        [System.Xml.Serialization.XmlIgnore]
        public CommandThread Commands
        {
            get
            {
                return this.commands ?? (this.commands = new CommandThread(DataService.Instance, this));
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView DigitalInputs { get; private set; }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView Probes { get; private set; }

        /// <summary>
        /// Gets the lights.
        /// </summary>
        /// <value>The lights.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView Lights { get; private set; }

        /// <summary>
        /// Gets the dosing pumps.
        /// </summary>
        /// <value>The dosing pumps.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView DosingPumps { get; private set; }

        /// <summary>
        /// Gets the current pumps.
        /// </summary>
        /// <value>The current pumps.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView CurrentPumps { get; private set; }

        /// <summary>
        /// Gets the level sensors.
        /// </summary>
        /// <value>The level sensors.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView LevelSensors { get; private set; }

        /// <summary>
        /// Gets the S ports.
        /// </summary>
        /// <value>The S ports.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView SPorts { get; private set; }

        /// <summary>
        /// Gets the L ports.
        /// </summary>
        /// <value>The L ports.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView LPorts { get; private set; }

        /// <summary>
        /// Gets the S ports.
        /// </summary>
        /// <value>The S ports.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ListCollectionView UserValues { get; private set; }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <value>The sensors.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ReadOnlyCollection<Probe> ProbeCollection
        {
            get
            {
                Collection<Probe> tempCollection = new Collection<Probe>();
                foreach (Probe item in this.Probes)
                {
                    tempCollection.Add(item);
                }

                return new ReadOnlyCollection<Probe>(tempCollection);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [sent alarm email].
        /// </summary>
        /// <value><c>true</c> if [sent alarm email]; otherwise, <c>false</c>.</value>
        [System.Xml.Serialization.XmlIgnore]
        public bool SentAlarmEmail { get; set; }

        /// <summary>
        /// Updates the alarm.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateInfoValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Info.SoftwareVersion == 0)
            {
                this.UpdateInfo(basicProtocol, progress);
            }
            else
            {
                Info.MoonPhase = basicProtocol.MoonPhase;
                this.UpdateAlarm(basicProtocol.Alarm, progress);
                this.UpdateOperationMode(basicProtocol.OpMode, progress);
                this.UpdateReminders(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the operation mode.
        /// </summary>
        /// <param name="operationMode">The operation mode.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateOperationMode(OperationMode operationMode, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Update Operation Mode";
            }

            if (Info.OperationMode != operationMode)
            {
                Info.OperationMode = operationMode;
                Logger.Instance.Log(new LogMessage(0, Language.GetResource("strOperationModechanged") + Language.GetFrendyName(operationMode)));
            }
        }

        /// <summary>
        /// Updates the alarm.
        /// </summary>
        /// <param name="alarmState">State of the alarm.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateAlarm(CurrentState alarmState, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Update Alarm State";
            }

            if (Info.Alarm != alarmState)
           {
                Info.Alarm = alarmState;
                Logger.Instance.Log(new LogMessage(alarmState == CurrentState.On ? 1 : 0, "System Alarm is now " + alarmState));
           }
        }

        /// <summary>
        /// Updates the reminders.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateReminders(IProfilux basicProtocol, IUpdateProgress progress)
        {
            foreach (Reminder reminder in Info.Reminders)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Update Reminder " + reminder.Text;
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DateTime? nextReminder = basicProtocol.GetNextReminder(reminder.Index);
                if (nextReminder.HasValue)
                {
                    reminder.Next = nextReminder.Value;
                    if (DateTime.Now > reminder.Next)
                    {
                        if (!reminder.IsOverdue)
                        {
                            reminder.IsOverdue = true;
                            Logger.Instance.Log(new LogMessage(2, Language.GetResource("strReminder") + " " + reminder.Text + " " + Language.GetResource("strIsOverdue")));
                        }
                    }
                    else
                    {
                        if (reminder.IsOverdue)
                        {
                            reminder.IsOverdue = false;
                            Logger.Instance.Log(
                                new LogMessage(
                                    0,
                                    Language.GetResource("strReminder") + " " + reminder.Text + " " + Language.GetResource("strIsNotOverdue")));
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Updates the info.
        /// </summary>
        /// <param name="basicProtocol">
        /// The basic protocol.
        /// </param>
        /// <param name="progress">
        /// The progress.
        /// </param>
        public void UpdateInfo(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.ProgressText = "Getting Controler Information";
            }

            Info.SoftwareVersion = basicProtocol.Version;
            Info.ProductId = basicProtocol.ProductId;
            Info.SerialNumber = basicProtocol.SerialNumber;
            Info.SoftwareDate = basicProtocol.SoftwareDate;
            Info.DeviceAddress = basicProtocol.DeviceAddress;
            if (Info.IsP3)
            {
                this.Connection.WebServerPort = basicProtocol.WebServerPort;
            }

            Info.Latitude = basicProtocol.Latitude;
            Info.Longitude = basicProtocol.Longitude;
            Info.MoonPhase = basicProtocol.MoonPhase;
            this.UpdateAlarm(basicProtocol.Alarm, progress);
            this.UpdateOperationMode(basicProtocol.OpMode, progress);

            for (int i = 0; i < basicProtocol.ReminderCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Reminder" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DateTime? nextReminder = basicProtocol.GetNextReminder(i);

                int index = i;
                Reminder reminder = Info.Reminders.FirstOrDefault(r => r.Index == index);

                if (nextReminder.HasValue)
                {
                    if (reminder == null)
                    {
                        reminder = new Reminder
                                       {
                                           Index = i
                                       };
                        Info.Reminders.Add(reminder);
                    }

                    // to reduce spam from the e-mail check to see if we have already sent the e-mail
                    if (reminder.Next != nextReminder)
                    {
                        reminder.SentMail = false;
                    }

                    reminder.Next = nextReminder.Value;
                    if (DateTime.Now > reminder.Next)
                    {
                        if (!reminder.IsOverdue)
                        {
                            reminder.IsOverdue = true;
                            Logger.Instance.Log(new LogMessage(2, Language.GetResource("strReminder") + " " + reminder.Text + Language.GetResource("strIsOverdue")));
                        }
                    }
                    else
                    {
                        if (reminder.IsOverdue)
                        {
                            reminder.IsOverdue = false;
                            Logger.Instance.Log(new LogMessage(0, Language.GetResource("strReminder") + " " + reminder.Text + Language.GetResource("strIsNotOverdue")));
                        }
                    }

                    reminder.Text = basicProtocol.GetReminderText(i);
                    reminder.Period = basicProtocol.GetReminderPeriod(i);
                    reminder.IsRepeating = basicProtocol.GetReminderRepeats(i);
                }
                else
                {
                    if (reminder != null)
                    {
                        Info.Reminders.Remove(reminder);
                    }
                }
            }
        }

        /// <summary>
        /// Updates the probes.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <param name="skipNonEssntals">if set to <c>true</c> [skip non essntals].</param>
        public void UpdateProbes(IProfilux basicProtocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            for (int i = 0; i < basicProtocol.SensorCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Sensor" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                SensorType type;
                SensorMode mode = SensorMode.Normal;
                bool active = false;
                try
                {
                    type = basicProtocol.GetSensorType(i);
                    if (type != SensorType.None && type != SensorType.Free)
                    {
                        mode = basicProtocol.GetSensorMode(i);
                        active = basicProtocol.GetSensorActive(i);
                    }
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                int i1 = i;
                Probe probe = (Probe) this.Items.FirstOrDefault(p => p is Probe && ((Probe) p).Index == i1);

                string sensorId = basicProtocol.GetSensorId(i, type);

                if (type != SensorType.None && type != SensorType.Free && (active || this.Connection.GetAll) && (mode == SensorMode.Normal || this.Connection.GetAll))
                {
                    if (probe == null)
                    {
                        probe = new Probe
                                    {
                                        Format = basicProtocol.GetSensorFormat(i), 
                                        Id = sensorId, 
                                        Index = i, 
                                        SensorType = type,
                                        SensorMode = mode
                                    };
                        this.Items.Add(probe);
                        OnPropertyChanged("Probes");
                    }
                    else
                    {
                        probe.Format = basicProtocol.GetSensorFormat(i);
                        probe.SensorMode = mode;
                        probe.Id = sensorId;
                        probe.Index = i;
                        probe.SensorType = type;
                    }

                    if (!skipNonEssntals)
                    {
                        probe.NominalValue = probe.ConvertFromInt(basicProtocol.GetSensorNominalValue(i));
                        probe.AlarmDeviation = probe.ConvertFromInt(basicProtocol.GetSensorAlarmDeviation(i));
                        probe.AlarmEnable = basicProtocol.GetSensorAlarmEnable(i);
                        UpdateProbeOperationalHours(basicProtocol, probe);
                    }

                    probe.SetValue(basicProtocol.GetSensorValue(i));
                    CurrentState alarm = basicProtocol.GetSensorAlarm(i);
                    if (alarm != probe.IsAlarmOn)
                    {
                        probe.IsAlarmOn = alarm;
                        Logger.Instance.Log(new LogMessage(alarm == CurrentState.On ? 1 : 0, "Sensor " + probe.DisplayName + " Alarm is now " + alarm));
                    }
                }
                else
                {
                    if (probe != null)
                    {
                        this.Items.Remove(probe);
                        OnPropertyChanged("Probes");
                    }
                }
            }
        }

        /// <summary>
        /// Updates the probe operational hours.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="probe">The probe.</param>
        private void UpdateProbeOperationalHours(IProfilux basicProtocol, Probe probe)
        {
            int hours = basicProtocol.GetProbeOperationHours(probe.Index);

            if (probe.EnableMaxOperationHours &&
                (!probe.IsOverMaxOperationHours && probe.MaxOperationHours < (hours / 60.0)))
            {
                Logger.Instance.Log(new LogMessage(2, string.Format(Language.GetResource("strOpHoursOver"), probe.DisplayName)));
            }

            probe.OperationHours = hours;
        }

        /// <summary>
        /// Updates the light operation hours.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="light">The light.</param>
        private void UpdateLightOperationHours(IProfilux basicProtocol, Light light)
        {
            int hours = basicProtocol.GetLightOperationHours(light.Channel);

            if (light.EnableMaxOperationHours &&
                (!light.IsOverMaxOperationHours && light.MaxOperationHours < (hours / 60.0)))
            {
                Logger.Instance.Log(new LogMessage(2, string.Format(Language.GetResource("strOpHoursOver"), light.DisplayName)));
            }

            light.OperationHours = hours;
        }

        /// <summary>
        /// Updates the probe values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateProbeValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Probes.Count != 0)
            {
                foreach (Probe probe in this.Probes)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Sensor " + probe.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    probe.SetValue(basicProtocol.GetSensorValue(probe.Index));
                    CurrentState alarm = basicProtocol.GetSensorAlarm(probe.Index);
                    if (alarm != probe.IsAlarmOn)
                    {
                        probe.IsAlarmOn = alarm;
                        Logger.Instance.Log(new LogMessage(alarm == CurrentState.On ? 1 : 0, "Sensor " + probe.DisplayName + " Alarm is now " + alarm));
                    }

                    UpdateProbeOperationalHours(basicProtocol, probe);
                }
            }
            else
            {
                this.UpdateProbes(basicProtocol, progress, false);
            }
        }

        /// <summary>
        /// Gets the sensors.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the communication
        /// </exception>
        public void UpdateLevelSensors(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.LevelSenosrCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Level Sensor" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                LevelSensorOpertationMode mode;
                try
                {
                    mode = basicProtocol.GetLevelSensorMode(i);
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                try
                {
                    int i1 = i;
                    LevelSensor sensor =
                        (LevelSensor) this.Items.FirstOrDefault(p => p is LevelSensor && ((LevelSensor) p).Index == i1);

                    if (mode != LevelSensorOpertationMode.NotEnabled || this.Connection.GetAll)
                    {
                        if (sensor == null)
                        {
                            sensor = new LevelSensor 
                            { 
                                Id = basicProtocol.GetLevelId(i),
                                Index = i
                            };
                            this.Items.Add(sensor);
                            OnPropertyChanged("LevelSensors");
                        }

                        sensor.OpertationMode = mode;
                        LevelState state = basicProtocol.GetLevelSensorState(i);
                        if (state.Alarm != sensor.IsAlarmOn)
                        {
                            sensor.IsAlarmOn = state.Alarm;
                            Logger.Instance.Log(new LogMessage(state.Alarm == CurrentState.On ? 1 : 0, "Level Sensor " + sensor.DisplayName + " Alarm is now " + state.Alarm));
                        }

                        sensor.Value = state.State;
                    }
                    else
                    {
                        if (sensor != null)
                        {
                            this.Items.Remove(sensor);
                            OnPropertyChanged("LevelSensors");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2000, string.Format("Level Sensor {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the level sensors values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateLevelSensorsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.LevelSensors.Count == 0)
            {
                foreach (LevelSensor sensor in this.LevelSensors)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Level Sensor " + sensor.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateLevelSensorsValue(basicProtocol, sensor);
                }
            }
            else
            {
                this.UpdateLevelSensors(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the level sensors value.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="sensor">The sensor.</param>
        public void UpdateLevelSensorsValue(IProfilux basicProtocol, LevelSensor sensor)
        {
            LevelState state = basicProtocol.GetLevelSensorState(sensor.Index);
            if (state.Alarm != sensor.IsAlarmOn)
            {
                sensor.IsAlarmOn = state.Alarm;
                Logger.Instance.Log(new LogMessage(state.Alarm == CurrentState.On ? 1 : 0, "Level Sensor " + sensor.DisplayName + " Alarm is now " + state.Alarm));
            }

            sensor.Value = state.State;
        }

        /// <summary>
        /// Updates the digital inputs.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateDigitalInputs(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.DigitalInputCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Digital Input" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                DigitalInputFunction mode;
                try
                {
                    mode = basicProtocol.GetDigitalInputFunction(i);
                }
                catch (ErrorCodeException)
                {
                    continue;
                }

                try
                {
                    int i1 = i;
                    DigitalInput sensor = (DigitalInput)this.Items.FirstOrDefault(p => p is DigitalInput && ((DigitalInput)p).Index == i1);
                    if (mode != DigitalInputFunction.NotUsed || this.Connection.GetAll)
                    {
                        if (sensor == null)
                        {
                            sensor = new DigitalInput
                                         {
                                             Id = basicProtocol.GetDigtialInputId(i), 
                                             Index = i
                                         };
                            this.Items.Add(sensor);
                            OnPropertyChanged("DigitalInputs");
                        }

                        sensor.Function = mode;

                        sensor.Value = basicProtocol.GetDigitalInputState(i);
                    }
                    else
                    {
                        if (sensor != null)
                        {
                            this.Items.Remove(sensor);
                            OnPropertyChanged("DigitalInputs");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2001, string.Format("Digital Input {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the digital input values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateDigitalInputValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.DigitalInputs.Count == 0)
            {
                foreach (DigitalInput sensor in this.DigitalInputs)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Digital Input " + sensor.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    sensor.Value = basicProtocol.GetDigitalInputState(sensor.Index);
                }
            }
            else
            {
                this.UpdateDigitalInputs(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Gets the Devices.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the commuincation
        /// </exception>
        public void UpdateSPorts(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.SPortCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting S" + (i + 1) + " Port Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    PortMode mode = basicProtocol.GetSPortFunction(i);
                    int portNumber = i;
                    SPort port = (SPort)this.Items.FirstOrDefault(p => p is SPort && ((SPort)p).PortNumber == portNumber);

                    if (mode.DeviceMode != DeviceMode.AlwaysOff || this.Connection.GetAll)
                    {
                        if (port == null)
                        {
                            port = new SPort
                                       {
                                           Id = basicProtocol.GetSPortId(portNumber), 
                                           PortNumber = portNumber
                                       };
                            port.UpdateMode(mode, this.ProbeCollection);
                            this.Items.Add(port);
                            OnPropertyChanged("SPorts");
                        }
                        else
                        {
                            port.UpdateMode(mode, this.ProbeCollection);
                        }

                        port.Info = Info;
                        port.SetValue(basicProtocol.GetSPortValue(i));
                        port.Current = basicProtocol.GetSPortCurrent(i);
                    }
                    else
                    {
                        if (port != null)
                        {
                            this.Items.Remove(port);
                            OnPropertyChanged("SPorts");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2002, string.Format("SPort {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the S ports value.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateSPortsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.SPorts.Count != 0)
            {
                foreach (SPort port in this.SPorts)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Socket " + port.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    port.Info = Info;
                    this.UpdateSPortValue(basicProtocol, port);
                    port.Current = basicProtocol.GetSPortCurrent(port.PortNumber);
                }
            }
            else
            {
                this.UpdateSPorts(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the S port value.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="port">The port.</param>
        public void UpdateSPortValue(IProfilux basicProtocol, SPort port)
        {
            port.SetValue(basicProtocol.GetSPortValue(port.PortNumber));
        }

        /// <summary>
        /// Gets the Devices.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <exception cref="ProtocolException">
        /// If there is a failure in the protocol
        /// </exception>
        /// <exception cref="ConnectionException">
        /// There is a failure in the communication
        /// </exception>
        public void UpdateLPorts(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.LPortCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting L" + (i + 1) + " Port Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int number = i;
                    PortMode mode = basicProtocol.GetLPortFunction(i);
                    LPort port = (LPort)this.Items.FirstOrDefault(p => p is LPort && ((LPort)p).PortNumber == number);
                    if (mode.DeviceMode != DeviceMode.AlwaysOff || this.Connection.GetAll)
                    {
                        if (port == null)
                        {
                            port = new LPort
                                       {
                                           Id = basicProtocol.GetLPortId(i), 
                                           PortNumber = i
                                       };
                            port.UpdateMode(mode, this.ProbeCollection);
                            this.Items.Add(port);
                            OnPropertyChanged("LPorts");
                        }
                        else
                        {
                            port.UpdateMode(mode, this.ProbeCollection);
                        }

                        port.Info = Info;
                        port.SetValue(basicProtocol.GetLPortValue(i));
                    }
                    else
                    {
                        if (port != null)
                        {
                            this.Items.Remove(port);
                            OnPropertyChanged("LPorts");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2003, string.Format("LPort {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the L port values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateLPortValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.LPorts.Count != 0)
            {
                foreach (LPort port in this.LPorts)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update 1-10V Port " + port.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    port.Info = Info;
                    port.SetValue(basicProtocol.GetLPortValue(port.PortNumber));
                }
            }
            else
            {
                this.UpdateLPorts(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Gets the item.
        /// </summary>
        /// <param name="itemId">The id.</param>
        /// <returns>and item with the given id</returns>
        public BaseInfo GetItem(string itemId)
        {
            return this.Items.FirstOrDefault(p => p.Id == itemId);
        }

        /// <summary>
        /// Changes the user values.
        /// </summary>
        public void ChangeUserValues()
        {
            OnPropertyChanged("UserValues");
        }

        /// <summary>
        /// Resets the reminder.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="reminder">The reminder.</param>
        public void ResetReminder(IProfilux basicProtocol, Reminder reminder)
        {
            if (reminder.IsRepeating)
            {
                basicProtocol.ResetReminder(reminder.Index, reminder.Period);
            }
            else
            {
                basicProtocol.ClearReminder(reminder.Index);
            }

            this.UpdateReminders(basicProtocol, null);
        }

        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <param name="skipNonEssntals">if set to <c>true</c> [skip non essntals].</param>
        public void UpdateLights(IProfilux basicProtocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            for (int i = 0; i < basicProtocol.LightCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Light" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int channel = i;
                    Light light = (Light)this.Items.FirstOrDefault(p => p is Light && ((Light)p).Channel == channel);

                    if (basicProtocol.IsLightActive(i))
                    {
                        if (light == null)
                        {
                            light = new Light
                                {
                                    Id = basicProtocol.GetLightId(channel), 
                                    Channel = channel, 
                                };
                            this.Items.Add(light);

                            OnPropertyChanged("Lights");
                        }

                        if (!skipNonEssntals)
                        {
                            light.IsDimmable = basicProtocol.IsLightDimmable(i);
                            this.UpdateLightOperationHours(basicProtocol, light);
                        }

                        light.Info = this.Info;
                        light.Value = basicProtocol.GetLightValue(i);
                    }
                    else
                    {
                        if (light != null)
                        {
                            this.Items.Remove(light);
                            OnPropertyChanged("Lights");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2002, string.Format("Light {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateLightsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Lights.Count != 0)
            {
                foreach (Light light in this.Lights)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Light " + light.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }
                    light.Info = this.Info;
                    this.UpdateLightValue(basicProtocol, light);
                    this.UpdateLightOperationHours(basicProtocol, light);
                }
            }
            else
            {
                this.UpdateLights(basicProtocol, progress, false);
            }
        }

        /// <summary>
        /// Updates the light value.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="light">The light.</param>
        public void UpdateLightValue(IProfilux basicProtocol, Light light)
        {
            light.Value = basicProtocol.GetLightValue(light.Channel);
        }


        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateCurrentPumps(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.CurrentPumpCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Current Pump" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int index = i;
                    var pump = (CurrentPump)this.Items.FirstOrDefault(p => p is CurrentPump && ((CurrentPump)p).Index == index);

                    if (basicProtocol.IsCurrentPumpAssinged(i))
                    {
                        if (pump == null)
                        {
                            pump = new CurrentPump { Id = basicProtocol.GetCurrentPumpId(index), Index = index, };
                            this.Items.Add(pump);

                            OnPropertyChanged("CurrentPumps");
                        }

                        pump.Info = this.Info;
                        pump.Value = basicProtocol.GetCurrentPumpValue(i);
                    }
                    else
                    {
                        if (pump != null)
                        {
                            this.Items.Remove(pump);
                            OnPropertyChanged("CurrentPumps");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2002, string.Format("CurrentPump {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateCurrentPumpValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.CurrentPumps.Count != 0)
            {
                foreach (CurrentPump pump in this.CurrentPumps)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Current Pump " + pump.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }
                    pump.Info = this.Info;
                    pump.Value = basicProtocol.GetCurrentPumpValue(pump.Index);
                }
            }
            else
            {
                this.UpdateLights(basicProtocol, progress, false);
            }
        }


        /// <summary>
        /// Updates the lights.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateDosingPumps(IProfilux basicProtocol, IUpdateProgress progress)
        {
            for (int i = 0; i < basicProtocol.TimerCount; i++)
            {
                if (progress != null)
                {
                    progress.IncrementProgress();
                    progress.ProgressText = "Getting Timer" + (i + 1) + " Information";
                    if (progress.StopProcessing)
                    {
                        return;
                    }
                }

                try
                {
                    int channel = i;
                    DosingPump pump = (DosingPump)this.Items.FirstOrDefault(p => p is DosingPump && ((DosingPump)p).Channel == channel);
                    TimerSettings settings = basicProtocol.GetTimerSettings(i);

                    if (settings.Mode == TimerMode.AutoDosing)
                    {
                        if (pump == null)
                        {
                            pump = new DosingPump
                            {
                                Id = basicProtocol.GetDousingPumpId(channel),
                                Channel = channel
                            };
                            this.Items.Add(pump);

                            OnPropertyChanged("DosingPumps");
                        }

                        pump.Settings = settings;
                        pump.Rate = basicProtocol.GetDosingRate(i);
                        pump.PerDay = settings.SwitchingCount;
                    }
                    else
                    {
                        if (pump != null)
                        {
                            this.Items.Remove(pump);
                            OnPropertyChanged("DosingPumpss");
                        }
                    }
                }
                catch (ErrorCodeException ex)
                {
                    Logger.Instance.Log(new LogMessage(2002, string.Format("Timer {0} not supported code {1}", i, ex.ErrorCode)) { Exception = ex });
                }
            }
        }

        /// <summary>
        /// Updates the update lights values.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="progress">The progress.</param>
        public void UpdateDosingPumpsValues(IProfilux basicProtocol, IUpdateProgress progress)
        {
            if (this.Lights.Count != 0)
            {
                foreach (DosingPump pump in this.DosingPumps)
                {
                    if (progress != null)
                    {
                        progress.IncrementProgress();
                        progress.ProgressText = "Update Dosing Pump " + pump.DisplayName;
                        if (progress.StopProcessing)
                        {
                            return;
                        }
                    }

                    this.UpdateDosingPumpsValue(basicProtocol, pump);
                }
            }
            else
            {
                this.UpdateDosingPumps(basicProtocol, progress);
            }
        }

        /// <summary>
        /// Updates the dosing pumps value.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="pump">The pump.</param>
        public void UpdateDosingPumpsValue(IProfilux basicProtocol, DosingPump pump)
        {
            pump.Rate = basicProtocol.GetDosingRate(pump.Channel);

            TimerSettings settings = basicProtocol.GetTimerSettings(pump.Channel);
            pump.Settings = settings;
            pump.PerDay = settings.SwitchingCount;
        }

        /// <summary>
        /// Updates the dosing rate.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="pump">The pump.</param>
        /// <param name="rate">The rate.</param>
        /// <param name="perDay">The per day.</param>
        public void UpdateDosingRate(IProfilux basicProtocol, DosingPump pump, int rate, int perDay)
        {
            basicProtocol.UpdateDosingRate(pump.Channel, rate);
            TimerSettings settings = pump.Settings;
            settings.SwitchingCount = perDay;
            basicProtocol.UpdateTimerSettings(pump.Channel, settings);
            this.UpdateDosingPumpsValue(basicProtocol, pump);
        }

        /// <summary>
        /// Clears the level alarm.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="levelSensor">The level sensor.</param>
        public void ClearLevelAlarm(IProfilux basicProtocol, LevelSensor levelSensor)
        {
            basicProtocol.ClearLevelAlarm(levelSensor.Index);
            this.UpdateLevelSensorsValue(basicProtocol, levelSensor);
            this.UpdateAlarm(basicProtocol.Alarm, null);
        }

        /// <summary>
        /// Waters the change.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="levelSensor">The level sensor.</param>
        public void WaterChange(IProfilux basicProtocol, LevelSensor levelSensor)
        {
            basicProtocol.WaterChange(levelSensor.Index);
        }

        /// <summary>
        /// Sets the state of the socket.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="port">The port.</param>
        /// <param name="value">if set to <c>true</c> [value].</param>
        public void SetSocketState(IProfilux basicProtocol, SPort port, bool value)
        {
            basicProtocol.SetSocketState(port.PortNumber, value);
        }

        /// <summary>
        /// Sets the state of the light.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="light">The light.</param>
        /// <param name="value">The value.</param>
        public void SetLightState(IProfilux basicProtocol, Light light, double value)
        {
            basicProtocol.SetLightValue(light.Channel, value);
        }

        /// <summary>
        /// Updates all.
        /// </summary>
        /// <param name="protocol">The protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <param name="skipNonEssntals">if set to <c>true</c> [skip non essntals].</param>
        /// <returns>True if all was updated</returns>
        public bool UpdateAll(IProfilux protocol, IUpdateProgress progress, bool skipNonEssntals)
        {
            if (progress != null)
            {
                progress.StopProcessing = false;
                progress.DisplayProgress = true;
                progress.ProgressText = "Update Started";
                double steps = protocol.ReminderCount +
                               protocol.SensorCount +
                               protocol.LevelSenosrCount +
                               protocol.LPortCount +
                               protocol.SPortCount +
                               protocol.DigitalInputCount +
                               protocol.TimerCount +
                               protocol.LightCount;
                progress.SetProgressSteps(steps);
            }

            try
            {
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateInfo(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateProbes(protocol, progress, skipNonEssntals);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLevelSensors(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateSPorts(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLPorts(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDigitalInputs(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDosingPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLights(protocol, progress, skipNonEssntals);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateCurrentPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.Info.LastUpdate = DateTime.Now;
            }
            finally
            {
                if (progress != null)
                {
                    progress.DisplayProgress = false;
                }
            }

            return true;
        }

        /// <summary>
        /// Updates the values.
        /// </summary>
        /// <param name="protocol">The protocol.</param>
        /// <param name="progress">The progress.</param>
        /// <returns>retuns True if the values were updated</returns>
        public bool UpdateValues(IProfilux protocol, IUpdateProgress progress)
        {
            if (progress != null)
            {
                progress.StopProcessing = false;
                progress.DisplayProgress = true;
                progress.ProgressText = "Log Data Started";
                double steps = this.Probes.Count +
                               this.LevelSensors.Count +
                               this.SPorts.Count +
                               this.LPorts.Count +
                               this.Lights.Count +
                               this.DosingPumps.Count +
                               this.Info.Reminders.Count;
                progress.SetProgressSteps(steps);
            }

            try
            {
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateInfoValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateProbeValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLevelSensorsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateSPortsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLPortValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDigitalInputValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateDosingPumpsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateLightsValues(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.UpdateCurrentPumps(protocol, progress);
                if (progress != null && progress.StopProcessing)
                {
                    return false;
                }

                this.Info.LastUpdate = DateTime.Now;
            }
            finally
            {
                if (progress != null)
                {
                    progress.DisplayProgress = false;
                }
            }

            return true;
        }

#if PocketPC != true
        /// <summary>
        /// Maintenances this instance.
        /// </summary>
        public void Maintenance()
        {
            this.Commands.SendMaintenance(Info.OperationMode == OperationMode.Maintenance);
        }

        /// <summary>
        /// Feeds this instance.
        /// </summary>
        public void Feed()
        {
            this.Commands.SendFeed(true);
        }

        /// <summary>
        /// Manuals the sockets.
        /// </summary>
        private void ManualSockets()
        {
            this.Commands.SendOperationMode(Info.OperationMode);
        }

        /// <summary>
        /// Manuals the sockets.
        /// </summary>
        private void ManualIllumination()
        {
            this.Commands.SendOperationMode(Info.OperationMode);
        }

#endif

        /// <summary>
        /// Resets the operational hours.
        /// </summary>
        /// <param name="basicProtocol">The basic protocol.</param>
        /// <param name="baseInfo">The base info.</param>
        public void ResetOperationalHours(IProfilux basicProtocol, BaseInfo baseInfo)
        {
            if (baseInfo is Light)
            {
                Light light = (Light)baseInfo;
                basicProtocol.SetLightOperationHours(light.Channel, 0);
                this.UpdateLightOperationHours(basicProtocol, light);
            }
            else if (baseInfo is Probe)
            {
                Probe probe = (Probe)baseInfo;
                basicProtocol.SetProbeOperationHours(probe.Index, 0);
                this.UpdateProbeOperationalHours(basicProtocol, probe);
            }
        }
    }
}
