﻿namespace RedPoint.ReefStatus.Common.ProfiLux
{
    /// <summary>
    /// The digital input.
    /// </summary>
    public class DigitalInput : SensorInfo
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DigitalInput"/> class.
        /// </summary>
        public DigitalInput()
        {
            SensorType = SensorType.DigitalInput;
            this.DefaultUnits = Language.GetResource("strState");
        }

        /// <summary>
        /// Gets the double value.
        /// </summary>
        /// <value>The double value.</value>
        public override double DoubleValue
        {
            get
            {
                if (this.Value != null)
                {
                    return (int) this.Value;
                }

                return 0;
            }
        }

        public override object ConvertedValue
        {
            get { return this.Value; }
        }


        private DigitalInputFunction function;

        /// <summary>
        /// Gets or sets the function.
        /// </summary>
        /// <value>The function.</value>
        public DigitalInputFunction Function
        {
            get
            {
                return this.function;
            }

            set
            {
                if (this.function != value)
                {
                    OnPropertyChanged("Function");
                    this.function = value;
                }
            }
        }

        /// <summary>
        /// Gets the mode.
        /// </summary>
        /// <value>The mode.</value>
        public override string Mode
        {
            get
            {
                return Language.GetFrendyName(this.Function);
            }
        }

        /// <summary>
        /// Gets the value with units.
        /// </summary>
        /// <value>The value with units.</value>
        public override string ValueWithUnits
        {
            get { return Language.GetFrendyName(this.Value); }
        }
    }
}