﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Reminder.cs" company="RedpointGames">
//   2010
// </copyright>
// <summary>
//   The Controlers Reminders
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace RedPoint.ReefStatus.Common.ProfiLux
{
    using System;
#if (PocketPC != true && Mono != true)
    using System.Windows.Input;
#endif
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;
    

    /// <summary>
    /// The Controlers Reminders
    /// </summary>
    public class Reminder : ViewModelBase
    {
        /// <summary>
        /// The index.
        /// </summary>
        private int index;

        /// <summary>
        /// The is repeating.
        /// </summary>
        private bool isRepeating;

        /// <summary>
        /// The next.
        /// </summary>
        private DateTime next;

        /// <summary>
        /// The period.
        /// </summary>
        private int period;

        /// <summary>
        /// The sent mail.
        /// </summary>
        private bool sentMail;

        /// <summary>
        /// The text.
        /// </summary>
        private string text;

        /// <summary>
        /// if the remider is overdew
        /// </summary>
        private bool isOverdue;

#if (PocketPC != true && Mono != true)
        private ICommand resetCommand;
#endif

        /// <summary>
        /// Gets or sets a value indicating whether this instance is overdue.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is overdue; otherwise, <c>false</c>.
        /// </value>
        public bool IsOverdue
        {
            get
            {
                return this.isOverdue;
            }

            set
            {
                if (this.isOverdue != value)
                {
                    this.isOverdue = value;
                    OnPropertyChanged("IsOverdue");
                }
            }
        }

        #if (PocketPC != true && Mono != true)
        /// <summary>
        /// Gets the reset command.
        /// </summary>
        /// <value>The reset command.</value>
        [System.Xml.Serialization.XmlIgnore]
        public ICommand ResetCommand
        {
            get
            {
                if(this.resetCommand == null)
                {
                    this.resetCommand = new DelegateCommand(Reset);
                }

                return this.resetCommand;
            }
        }
#endif
        /// <summary>
        /// Gets or sets a value indicating whether [sent mail].
        /// </summary>
        /// <value><c>true</c> if [sent mail]; otherwise, <c>false</c>.</value>
        public bool SentMail
        {
            get
            {
                return this.sentMail;
            }

            set
            {
                if (this.sentMail != value)
                {
                    this.sentMail = value;
                    OnPropertyChanged("SentMail");
                }
            }
        }

        /// <summary>
        /// Gets or sets the next.
        /// </summary>
        /// <value>The next.</value>
        public DateTime Next
        {
            get
            {
                return this.next;
            }

            set
            {
                if (this.next != value)
                {
                    this.next = value;
                    OnPropertyChanged("Next");
                }
            }
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text
        {
            get
            {
                return this.text;
            }

            set
            {
                if (this.text != value)
                {
                    this.text = value;
                    OnPropertyChanged("Text");
                }
            }
        }

        /// <summary>
        /// Gets or sets Index.
        /// </summary>
        public int Index
        {
            get
            {
                return this.index;
            }

            set
            {
                if (this.index != value)
                {
                    this.index = value;
                    OnPropertyChanged("Index");
                }
            }
        }

        /// <summary>
        /// Gets or sets the period.
        /// </summary>
        /// <value>The period.</value>
        public int Period
        {
            get
            {
                return this.period;
            }

            set
            {
                if (this.period != value)
                {
                    this.period = value;
                    OnPropertyChanged("Period");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is repeating.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is repeating; otherwise, <c>false</c>.
        /// </value>
        public bool IsRepeating
        {
            get
            {
                return this.isRepeating;
            }

            set
            {
                if (this.isRepeating != value)
                {
                    this.isRepeating = value;
                    OnPropertyChanged("IsRepeating");
                }
            }
        }

#if (PocketPC != true)
        /// <summary>
        /// Resets this instance.
        /// </summary>
        public void Reset()
        {
            ReefStatusSettings.Instance.GetControler(this).Commands.SendResetReminder(this);
        }
#endif
    }
}
