﻿// <copyright file="SPort.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus.Common.ProfiLux
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;

    /// <summary>
    /// Socket Status
    /// </summary>
    public class SPort : DeviceInfo
    {
        private double current;

        private bool displayCurrentGraph;

        /// <summary>
        /// Gets the double value.
        /// </summary>
        /// <value>The double value.</value>
        public override double DoubleValue
        {
            get
            {
                return this.Value != null ? (int)this.Value : 0;
            }
        }

        public override object ConvertedValue
        {
            get { return this.Value; }
        }

        /// <summary>
        /// Updates the mode.
        /// </summary>
        /// <param name="mode">The mode.</param>
        /// <param name="probes">The probes.</param>
        public void UpdateMode(PortMode mode, ReadOnlyCollection<Probe> probes)
        {
            this.DeviceMode = mode.DeviceMode;
            this.Port = mode.Port;
            this.Blackout = mode.BlackOut;
            this.Invert = mode.Invert;
            this.ModeString = this.SocketModeString(probes);
            this.DefaultUnits = "State";
        }

        /// <summary>
        /// Gets the graph id.
        /// </summary>
        /// <value>The graph id.</value>
        public override string GraphId
        {
            get
            {
                return this.GraphCurrent? this.CurrentId : this.Id;
            }
        }

        /// <summary>
        /// Gets the current id.
        /// </summary>
        /// <value>The current id.</value>
        public string CurrentId
        {
            get
            {
                return this.Id + "_Current";
            }
        }

        private bool graphCurrent;

        /// <summary>
        /// Gets or sets a value indicating whether [graph current].
        /// </summary>
        /// <value><c>true</c> if [graph current]; otherwise, <c>false</c>.</value>
        public bool GraphCurrent
        {
            get
            {
                return this.graphCurrent;
            }
            set
            {
                if (this.graphCurrent != value)
                {
                    this.graphCurrent = value;
                    this.OnPropertyChanged("GraphCurrent");
                    this.OnPropertyChanged("GraphId");
                }
            }
        }

        /// <summary>
        /// Gets the value with units.
        /// </summary>
        /// <value>The value with units.</value>
        public override string ValueWithUnits
        {
            get 
            {
                return Language.GetFrendyName(this.Value); 
            }
        }

        /// <summary>
        /// Converts the value.
        /// </summary>
        /// <param name="value">The value.</param>
        public void SetValue(object value)
        {
            this.Value = (CurrentState) value;
        }

#if (PocketPC != true)
        /// <summary>
        /// Gets or sets a value indicating whether this instance is active.
        /// </summary>
        /// <value><c>true</c> if this instance is active; otherwise, <c>false</c>.</value>
        [System.Xml.Serialization.XmlIgnore]
        public bool IsActive
        {
            get
            {
                if (Value == null)
                    return false;

                return (CurrentState)this.Value == CurrentState.On;
            }

            set
            {
                if (((CurrentState)this.Value == CurrentState.On) != value)
                {
                    Commands.SendSocketState(this, value);
                }
            }
        }

        public double Current
        {
            get
            {
                return this.current;
            }

            set
            {
                if (this.current != value)
                {
                    this.current = value;
                    this.OnPropertyChanged("Current");
                }
            }
        }

        public bool DisplayCurrentGraph
        {
            get
            {
                return this.displayCurrentGraph;
            }

            set
            {
                if (this.displayCurrentGraph != value)
                {
                    this.displayCurrentGraph = value;
                    this.OnPropertyChanged("DisplayCurrentGraph");
                }
            }
        }
#endif

        /// <summary>
        /// Gets Sockets the mode.
        /// </summary>
        /// <param name="probes">The probes.</param>
        /// <returns>the socket mode string</returns>
        private string SocketModeString(IEnumerable<Probe> probes)
        {
            string invertText = string.Empty;
            if (this.Invert)
            {
                invertText = " (" + Language.GetResource("strInvert") + ")";
            }

            string deviceMode = Language.GetFrendyName(this.DeviceMode);

            if (this.IsUsePort)
            {
                return deviceMode + " " + this.Port + invertText;
            }

            if (this.IsProbe)
            {
                Probe probe = probes.FirstOrDefault(p => p.Index == (this.Port - 1));
                if (probe != null)
                {
                    return probe.Id + " " + deviceMode + invertText;
                }

                return Language.GetResource("strSensor") + this.Port + " " + deviceMode + invertText;
            }

            return deviceMode + invertText;
        }
    }
}
