﻿namespace RedPoint.ReefStatus.Common.UI
{
    using System.Collections.ObjectModel;
    using System.Threading;

    using RedPoint.ReefStatus.Common.Core;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;

    public class ImportFromProfilux
    {
        /// <summary>
        /// Starts the specified callback.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <param name="controler">The controler.</param>
        public static void Start(IProgressCallback callback, Controler controler)
        {
            new Thread(() =>
                    {
                        if (callback != null)
                        {
                            lock (callback.Lock)
                            {
                                try
                                {
                                    callback.Begin(Language.GetResource("strImport"));

                                    Collection<ItemDataRow> data = controler.Commands.GetDataPoints(callback);

                                    if (callback.IsAborting)
                                    {
                                        return;
                                    }

                                    using (IDataAccess access = ReefStatusSettings.Instance.Logging.Connection.Create())
                                    {
                                        access.AddLog(data, callback, controler.Id);
                                    }
                                }
                                catch (ReefStatusException ex)
                                {
                                    Logger.Instance.LogError(ex);
                                }
                                finally
                                {
                                    callback.End();
                                }
                            }
                        }
                    }).Start();
        }
    }
}
