﻿namespace RedPoint.ReefStatus.Gui.Views
{
    using System.Windows;

    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Gui.ViewModels;

    /// <summary>
    /// Interaction logic for StatusControl.xaml
    /// </summary>
    public partial class StatusView
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="StatusView"/> class.
        /// </summary>
        public StatusView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// When overridden in a derived class, is invoked whenever application code or internal processes call <see cref="M:System.Windows.FrameworkElement.ApplyTemplate"/>.
        /// </summary>
        public override void OnApplyTemplate()
        {
            if (DataContext != null)
            {
                DataContext = new StatusViewModel((Controler)DataContext);
            }
        }

        /// <summary>
        /// Handles the MouseDoubleClick event of the ListView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Input.MouseButtonEventArgs"/> instance containing the event data.</param>
        private void ListView_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (((StatusViewModel)DataContext).ShowGraphCommand.CanExecute(null))
            {
                ((StatusViewModel) DataContext).ShowGraphCommand.Execute(null);
            }
        }

        /// <summary>
        /// Handles the Click event of the Settings control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void Settings_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            DataSettingsView view = new DataSettingsView
                                        {
                                            DataContext = ((StatusViewModel) this.DataContext).CurrentSelection
                                        };
            view.ShowDialog();
        }

        /// <summary>
        /// Handles the Click event of the UpdateUserItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void UpdateUserItem_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            if (((StatusViewModel)DataContext).UpdateCommand.CanExecute(null))
            {
                ((StatusViewModel)DataContext).UpdateCommand.Execute(null);
                UpdateUserValueView view = new UpdateUserValueView { DataContext = this.DataContext };
                view.ShowDialog();
            }
        }

        private void UpdateDosing_Click(object sender, RoutedEventArgs e)
        {
            if (((StatusViewModel)DataContext).UpdateDosingCommand.CanExecute(null))
            {
                ((StatusViewModel)DataContext).UpdateDosingCommand.Execute(null);
                UpdateDosingView view = new UpdateDosingView {DataContext = this.DataContext};
                view.ShowDialog();
            }
        }
    }
}