﻿namespace RedPoint.ReefStatus.Mobile
{
    using System;
    using System.Drawing;
    using System.Threading;
    using System.Windows.Forms;
    using Microsoft.Win32;
    using Common;
    using Common.ProfiLux;
    using Common.Settings;

    /// <summary>
    /// 
    /// </summary>
    public partial class MainView : Form, IUpdateProgress
    {
        /// <summary>
        /// the controler
        /// </summary>
        private readonly Controler controler = new Controler();

        private object GetValue(string value, object defaultValue)
        {
            object item = Registry.GetValue("HKEY_CURRENT_USER" + "\\SOFTWARE\\RedPoint\\Reef Status", value, defaultValue);
            return item ?? defaultValue;
        }

        public MainView()
        {

            InitializeComponent();

            this.password.Text = (string) this.GetValue("password", string.Empty);
            this.port.Value = (int)this.GetValue("Port", 8081);
            this.address.Text = (string)this.GetValue("Address", string.Empty);
            this.connectionType.SelectedIndex = ((ConnectionType)(int)this.GetValue("ConnectionType", (int)ConnectionType.Network)) == ConnectionType.Network ? 0 : 1;

            this.UpdateConnectionData();

            this.RefreshData();
        }


        private ConnectionSettings Connection
        {
            get
            {
                return new ConnectionSettings
                                     {
                                         Password = password.Text,
                                         Address = address.Text,
                                         Port = (int)port.Value,
                                         ConnectionType = connectionType.SelectedIndex == 0 ? ConnectionType.Network : ConnectionType.Web
                                     };
            }
        }

        private void UpdateConnectionData()
        {
            if (connectionType.SelectedIndex == 0)
            {
                this.password.Visible = false;
                this.passwordLabel.Visible = false;
            }
            else
            {
                this.password.Visible = true;
                this.passwordLabel.Visible = true;
            }
        }

        private void RefreshData()
        {
            if (this.Connection.Address != string.Empty)
            {
                ConnectionSettings connection = this.Connection;
                Registry.SetValue("HKEY_CURRENT_USER" + "\\SOFTWARE\\RedPoint\\Reef Status", "Password", connection.Password);
                Registry.SetValue("HKEY_CURRENT_USER" + "\\SOFTWARE\\RedPoint\\Reef Status", "Port", connection.Port);
                Registry.SetValue("HKEY_CURRENT_USER" + "\\SOFTWARE\\RedPoint\\Reef Status", "Address", connection.Address);
                Registry.SetValue("HKEY_CURRENT_USER" + "\\SOFTWARE\\RedPoint\\Reef Status", "ConnectionType", (int)connection.ConnectionType);

                Thread thread = new Thread(() =>
                {
                    IProfilux profilux = null;

                    try
                    {
                        profilux = Protocol.Connect(connection);

                        DisplayProgress = true;
                        double steps = profilux.ReminderCount +
                               profilux.SensorCount +
                               profilux.LevelSenosrCount +
                               profilux.LPortCount +
                               profilux.SPortCount +
                               profilux.DigitalInputCount +
                               profilux.TimerCount +
                               profilux.LightCount;
                        SetProgressSteps(steps);

                        this.controler.UpdateInfo(profilux, this);

                        this.Invoke(new Action(() =>
                                                   {
                                                       softwareVersion.Text = this.controler.Info.SoftwareVersion.ToString();
                                                       softwareDate.Text = this.controler.Info.SoftwareDate.ToShortDateString();
                                                       model.Text = this.controler.Info.Model;
                                                       serialNumber.Text = this.controler.Info.SerialNumber.ToString();
                                                       deviceAddress.Text = this.controler.Info.DeviceAddress.ToString();
                                                       alarm.Text = this.controler.Info.Alarm.ToString();
                                                       mode.Text = this.controler.Info.OperationMode.ToString();
                                                   }));
                        this.Invoke(new Action(() =>
                        {
                            reminderList.Items.Clear();
                            foreach (Reminder reminder in this.controler.Info.Reminders)
                            {
                                ListViewItem item = new ListViewItem();
                                if (reminder.IsOverdue)
                                {
                                    item.BackColor = Color.Yellow;
                                }

                                item.Text = reminder.Next.ToShortDateString();
                                reminderList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = reminder.Text });
                            }
                        }));

                        this.controler.UpdateProbes(profilux, this, true);

                        this.Invoke(new Action(() =>
                                                   {
                                                       probeList.Items.Clear();
                                                       foreach (Probe probe in this.controler.Probes)
                                                       {
                                                           ListViewItem item = new ListViewItem();
                                                           if (probe.IsAlarmOn == CurrentState.On)
                                                           {
                                                               item.BackColor = Color.Red;
                                                           }

                                                           item.Text = probe.DisplayName;
                                                           probeList.Items.Add(item);
                                                           item.SubItems.Add(new ListViewItem.ListViewSubItem
                                                                                 {
                                                                                     Text = probe.ValueWithUnits
                                                                                 });
                                                       }
                                                   }));

                        this.controler.UpdateLevelSensors(profilux, this);

                        this.Invoke(new Action(() =>
                        {
                            levelList.Items.Clear();
                            foreach (LevelSensor sensor in this.controler.LevelSensors)
                            {
                                ListViewItem item = new ListViewItem { Text = sensor.DisplayName };
                                if (sensor.IsAlarmOn == CurrentState.On)
                                {
                                    item.BackColor = Color.Red;
                                }

                                levelList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sensor.OpertationMode.ToString() });
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sensor.ValueWithUnits });
                            }
                        }));

                        this.controler.UpdateDigitalInputs(profilux, this);

                        this.Invoke(new Action(() =>
                        {
                            digitalList.Items.Clear();
                            foreach (DigitalInput sensor in this.controler.DigitalInputs)
                            {
                                ListViewItem item = new ListViewItem { Text = sensor.DisplayName };
                                digitalList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sensor.Function.ToString() });
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sensor.ValueWithUnits });
                            }
                        }));

                        this.controler.UpdateSPorts(profilux, this);

                        this.Invoke(new Action(() =>
                                                    {
                                                        sportList.Items.Clear();
                                                        foreach (SPort sport in this.controler.SPorts)
                                                        {
                                                            ListViewItem item = new ListViewItem { Text = sport.DisplayName };
                                                            sportList.Items.Add(item);
                                                            item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sport.ModeString });
                                                            item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = sport.ValueWithUnits });
                                                        }
                                                    }));

                        this.controler.UpdateLPorts(profilux, this);
                        this.Invoke(new Action(() =>
                        {
                            lportList.Items.Clear();
                            foreach (LPort lport in this.controler.LPorts)
                            {
                                ListViewItem item = new ListViewItem { Text = lport.DisplayName };
                                lportList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = lport.ModeString });
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = lport.ValueWithUnits });
                            }
                        }));

                        this.controler.UpdateDosingPumps(profilux, this);
                        this.Invoke(new Action(() =>
                        {
                            dousingList.Items.Clear();
                            foreach (DosingPump pump in this.controler.DosingPumps)
                            {
                                ListViewItem item = new ListViewItem { Text = pump.DisplayName };
                                dousingList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = pump.ValueWithUnits });
                            }
                        }));

                        this.controler.UpdateLights(profilux, this, true);
                        this.Invoke(new Action(() =>
                        {
                            lightList.Items.Clear();
                            foreach (Light light in this.controler.Lights)
                            {
                                ListViewItem item = new ListViewItem { Text = light.DisplayName };
                                lightList.Items.Add(item);
                                item.SubItems.Add(new ListViewItem.ListViewSubItem { Text = light.ValueWithUnits });
                            }
                        }));

                        this.controler.Info.LastUpdate = DateTime.Now;
                        this.Invoke(new Action(() =>
                        {
                            lastUpdate.Text = this.controler.Info.LastUpdate.ToString();
                        }));
                    }
                    catch (ReefStatusException ex)
                    {
                        Logger.Instance.LogError(ex);
                    }
                    finally 
                    {
                        DisplayProgress = false;
                        if (profilux != null)
                        {
                            profilux.Disconnect();
                        }
                    }
                });
                thread.Start();
            }
            else
            {
                tabControl1.SelectedIndex = tabControl1.TabPages.IndexOf(settingsPage);
            }
        }

        /// <summary>
        /// Handles the Click event of the refresh control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void refresh_Click(object sender, System.EventArgs e)
        {
            this.RefreshData();
        }

        /// <summary>
        /// Handles the Click event of the feedButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void feedButton_Click(object sender, System.EventArgs e)
        {
            ConnectionSettings connection = this.Connection;
            Thread thread = new Thread(() =>
                                           {
                                               IProfilux profilux = Protocol.Connect(connection);
                                               profilux.FeedPause(true);
                                               profilux.Disconnect();
                                           });
            thread.Start();
        }

        /// <summary>
        /// Handles the Click event of the thunderStorm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void thunderStorm_Click(object sender, System.EventArgs e)
        {
            ConnectionSettings connection = this.Connection;
            Thread thread = new Thread(() =>
                                           {
                                               IProfilux profilux = Protocol.Connect(connection);
                                               profilux.Thunderstorm(5);
                                               profilux.Disconnect();
                                           });
            thread.Start();
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the connectionType control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void connectionType_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            this.UpdateConnectionData();
        }

        #region IUpdateProgress Members

        public bool DisplayProgress
        {
            get
            {
                return refreshProgress.Visible;
            }
            
            set
            {
                this.Invoke(new Action(() =>
                {
                    refreshProgress.Visible = value;
                }));
            }
        }

        public bool StopProcessing
        {
            get; set;
        }

        public void SetProgressSteps(double steps)
        {
            this.Invoke(new Action(() =>
                                       {
                                           refreshProgress.Value = 0;
                                           refreshProgress.Maximum = (int)steps;
                                           refreshProgress.Minimum = 0;
                                       }));
        }

        public string ProgressText
        {
            get; set;
        }

        public void IncrementProgress()
        {
            this.Invoke(new Action(() =>
            {
                refreshProgress.Value++;
            }));
        }

        #endregion
    }
}