namespace Redpoint.ReefStatus.ScreenSaver
{
    using System;
    using System.Globalization;
    using System.Security.Permissions;
    using System.Windows;

    using RedPoint.ReefStatus.Common;
    using RedPoint.ReefStatus.Common.Localization;

    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App
    {

        static System.Reflection.Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            if (args.Name.StartsWith("System.Data.SQLite, ", StringComparison.OrdinalIgnoreCase))
            {
                string realName = IntPtr.Size == 8 ? "x64/System.Data.SQLite.dll" : "System.Data.SQLite.dll";
                return System.Reflection.Assembly.LoadFile(realName);
            }

            return null;
        }

        /// <summary>
        /// Called when [startup].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.StartupEventArgs"/> instance containing the event data.</param>
        [SecurityPermission(SecurityAction.Demand, Flags = SecurityPermissionFlag.ControlAppDomain)]
        public void OnStartup(object sender, StartupEventArgs e)
        {
            AppDomain.CurrentDomain.AssemblyResolve += CurrentDomain_AssemblyResolve;

            Dump.Handle();

            try
            {
                ResourceLoader.LoadFileResources(Current.Resources.MergedDictionaries, CultureInfo.CurrentCulture.Name);
            }
            catch (Exception)
            {
            }

            string[] args = e.Args;
            if (args.Length > 0)
            {
                // Get the 2 character command line argument
                string arg = args[0].ToLower(CultureInfo.InvariantCulture).Trim().Substring(0, 2);
                switch (arg)
                {
                    case "/c":
                        MessageBox.Show("The Reef Status Screen Saver uses your settings from Reef Status Application.", "Reef Status");
                        break;
                    case "/p":
                        // Don't do anything for preview
                        Application.Current.Shutdown();
                        break;
                    case "/s":
                        // Show screensaver form
                        this.ShowScreensaver();
                        break;
                    default:
                        Application.Current.Shutdown();
                        break;
                }
            }
            else
            {
                // If no arguments were passed in, show the screensaver
                this.ShowScreensaver();
            }
        }

        /// <summary>
        /// Shows screen saver by creating one instance of Window1 for each monitor.
        /// Note: uses WinForms's Screen class to get monitor info.
        /// </summary>
        void ShowScreensaver()
        {
            // creates window on primary screen
            MainWindowView primaryWindow = new MainWindowView
                {
                    WindowStartupLocation = WindowStartupLocation.Manual, 
                    WindowState = WindowState.Maximized
                };

            // creates window on other screens
            foreach (System.Windows.Forms.Screen screen in System.Windows.Forms.Screen.AllScreens)
            {
                if (screen == System.Windows.Forms.Screen.PrimaryScreen)
                {
                    continue;
                }

                MainWindowView window = new MainWindowView { WindowStartupLocation = WindowStartupLocation.Manual };
                System.Drawing.Rectangle location = screen.Bounds;

                // covers entire monitor
                window.Left = location.X - 7;
                window.Top = location.Y - 7;
                window.Width = location.Width + 14;
                window.Height = location.Height + 14;
            }

            // show non-primary screen windows
            foreach (Window window in System.Windows.Application.Current.Windows)
            {
                if (window != primaryWindow)
                {
                    window.Show();
                }
            }

            // shows primary screen window last
            primaryWindow.Show();
        }
    }
}