﻿// <copyright file="CustomGraphSettings.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System.Collections.ObjectModel;
    using System.Windows.Forms;
    using Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    /// <summary>
    /// Custom Graph settings form
    /// </summary>
    public partial class CustomGraphSettings : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="CustomGraphSettings"/> class.
        /// </summary>
        /// <param name="settings">The settings.</param>
        public CustomGraphSettings(ReefStatusSettings settings)
        {
            InitializeComponent();

            foreach (BaseInfo graph in settings.FirstControler.Items)
            {
                curveBox.Items.Add(graph, settings.CustomGraph.ShowCurves.Contains(graph.Id));
            }

            curveBox.CheckOnClick = true;

            rangeBox.Items.AddRange(new[] { "All", "Day", "Week", "Month", "Year" });
            rangeBox.SelectedIndex = (int)settings.CustomGraph.Range;
        }

        /// <summary>
        /// Gets the selected curves.
        /// </summary>
        /// <value>The selected curves.</value>
        public SafeObservableCollection<string> SelectedCurves
        {
            get
            {
                SafeObservableCollection<string> curves = new SafeObservableCollection<string>();

                foreach (BaseInfo item in curveBox.CheckedItems)
                {
                    curves.Add(item.Id);
                }

                return curves;
            }
        }

        /// <summary>
        /// Gets the range.
        /// </summary>
        /// <value>The range.</value>
        public GraphRange Range
        {
            get
            {
                return (GraphRange)rangeBox.SelectedIndex;
            }
        }
    }
}
