﻿// <copyright file="DataPointsView.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Windows.Forms;
    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;

    /// <summary>
    /// View the data points
    /// </summary>
    public partial class DataPointsView : Form
    {
        private DatabaseConnection connection;


        /// <summary>
        /// Initializes a new instance of the <see cref="DataPointsView"/> class.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="type">The type.</param>
        public DataPointsView(DatabaseConnection connection, BaseInfo item)
        {
            InitializeComponent();
            this.connection = connection;
            this.Item = item;
        }

        /// <summary>
        /// Gets the item.
        /// </summary>
        /// <value>The item.</value>
        public BaseInfo Item { get; private set; }

        /// <summary>
        /// Updates the data.
        /// </summary>
        public void UpdateData()
        {
            using (IDataAccess dataAccess = connection.Create())
            {
                BindingSource bs = new BindingSource();
                    bs.DataSource = dataAccess.GetDataPoints(Item.Id, Item.Controler.Id);
                    dataGridView1.DataSource = bs;
            }
        }

        /// <summary>
        /// Handles the Load event of the DataPointsView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DataPointsView_Load(object sender, EventArgs e)
        {
            UpdateData();
        }
    }
}
