﻿namespace RedPoint.ReefStatus
{
    using System;
    using System.Drawing;
    using System.Windows.Forms;

    public partial class Expander : UserControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Expander"/> class.
        /// </summary>
        public Expander()
        {
            InitializeComponent();
            IsExpanded = false;
        }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <value>The content.</value>
        public ListView Content
        {
            get
            {
                return content;
            }
        }

        /// <summary>
        /// Gets the header.
        /// </summary>
        /// <value>The header.</value>
        public CheckBox Header
        {
            get
            {
                return header;
            }
        }

        /// <summary>
        /// Gets or sets the header text.
        /// </summary>
        /// <value>The header text.</value>
        public string HeaderText
        {
            get
            {
                return header.Text;
            }

            set
            {
                header.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is expanded.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is expanded; otherwise, <c>false</c>.
        /// </value>
        public bool IsExpanded
        {
            get
            {
                return header.Checked;
            }

            set
            {
                header.Checked = value;
                content.Visible = value;
            }
        }

        /// <summary>
        /// Gets or sets the image.
        /// </summary>
        /// <value>The image.</value>
        public Image Image
        {
            get
            {
                return header.Image;
            }

            set
            {
                header.Image = value;
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the header control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void header_CheckedChanged(object sender, EventArgs e)
        {
            content.Visible = header.Checked;
        }

        /// <summary>
        /// Expands this instance.
        /// </summary>
        public void Expand()
        {
            IsExpanded = true;
        }
    }
}
