﻿namespace RedPoint.ReefStatus.DBTest
{
    using System;
    using System.Collections.ObjectModel;
    using System.Windows.Input;

    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;
    using RedPoint.ReefStatus.Common.UI.ViewModel;

    public class MainViewModel : ViewModelBase
    {
        private double newValue;

        private string type;

        private ICommand addToDatbaseCommand;

        private ICommand refreshCommand;

        public MainViewModel()
        {
            this.Settings = ReefStatusSettings.Instance;

            this.Types = new ObservableCollection<string>();
            this.Points = new ObservableCollection<DataPoint>();
            this.Refresh(string.Empty);
        }

        /// <summary>
        /// Gets or sets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public ReefStatusSettings Settings { get; private set; }


        /// <summary>
        /// Adds to database.
        /// </summary>
        public void AddToDatabase()
        {
            using (IDataAccess access = this.Settings.Logging.Connection.Create())
            {
                if (access != null)
                {
                    access.InsertItem(this.NewValue, DateTime.Now, this.Type, true, 0);
                }
            }

            this.Refresh(this.Type);
        }

        /// <summary>
        /// Gets or sets the new value.
        /// </summary>
        /// <value>The new value.</value>
        public double NewValue
        {
            get
            {
                return this.newValue;
            }

            set
            {
                if (value == this.newValue)
                {
                    return;
                }

                this.newValue = value;
                OnPropertyChanged("NewValue");
            }
        }

        /// <summary>
        /// Gets or sets the type.
        /// </summary>
        /// <value>The type.</value>
        public string Type
        {
            get
            {
                return this.type;
            }

            set
            {
                if (value == this.type)
                {
                    return;
                }

                this.type = value;
                OnPropertyChanged("Type");
            }
        }

        /// <summary>
        /// Gets the add to datbase command.
        /// </summary>
        /// <value>The add to datbase command.</value>
        public ICommand AddToDatbaseCommand
        {
            get
            {
                return this.addToDatbaseCommand ?? (this.addToDatbaseCommand = new DelegateCommand(this.AddToDatabase));
            }
        }

        /// <summary>
        /// Gets the add to datbase command.
        /// </summary>
        /// <value>The add to datbase command.</value>
        public ICommand RefreshCommand
        {
            get
            {
                return this.refreshCommand ?? (this.refreshCommand = new DelegateCommand<string>(this.Refresh));
            }
        }

        /// <summary>
        /// Gets or sets the types.
        /// </summary>
        /// <value>The types.</value>
        public ObservableCollection<string> Types { get;  private set; }

        /// <summary>
        /// Gets or sets the points.
        /// </summary>
        /// <value>The points.</value>
        public ObservableCollection<DataPoint> Points { get; private set; }

        /// <summary>
        /// Refreshes this instance.
        /// </summary>
        /// <param name="dataType">Type of the data.</param>
        private void Refresh(string dataType)
        {
            using (IDataAccess access = this.Settings.Logging.Connection.Create())
            {
                if (access != null)
                {
                    this.Types.Clear();
                    foreach (var s in access.GetTypes().Values)
                    {
                        this.Types.Add(s);
                    }

                    if (!string.IsNullOrEmpty(dataType))
                    {
                        this.Points.Clear();
                        foreach (var dataPoint in access.GetDataPoints(dataType, false, 0))
                        {
                            this.Points.Add(dataPoint);
                        }
                    }
                }
            }
        }
    }
}
