﻿using RedPoint.ReefStatus.Common.UI.ViewModel;

namespace RedPoint.ReefStatus.Common.Settings
{
    public class WebInterfaceSettings : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WebInterfaceSettings"/> class.
        /// </summary>
        public WebInterfaceSettings()
        {
            this.Enable = false;
            this.Port = 8081;
            this.Password = string.Empty;
            this.Protection = false;
        }

        private string password;

        /// <summary>
        /// Gets or sets the web password.
        /// </summary>
        /// <value>The web password.</value>
        public string Password
        {
            get { return password; }
            set
            {
                password = value;
                OnPropertyChanged("Password");
            }
        }

        private bool protection;

        /// <summary>
        /// Gets or sets a value indicating whether [web protection].
        /// </summary>
        /// <value><c>true</c> if [web protection]; otherwise, <c>false</c>.</value>
        public bool Protection
        {
            get { return protection; }
            set
            {
                protection = value;
                OnPropertyChanged("Protection");
            }
        }

        private bool enable;

        /// <summary>
        /// Gets or sets a value indicating whether [web interface enable].
        /// </summary>
        /// <value><c>true</c> if [web interface enable]; otherwise, <c>false</c>.</value>
        public bool Enable
        {
            get { return enable; }
            set
            {
                enable = value;
                OnPropertyChanged("Enable");
            }
        }

        private int port;

        /// <summary>
        /// Gets or sets the web interface port.
        /// </summary>
        /// <value>The web interface port.</value>
        public int Port
        {
            get { return port; }
            set
            {
                port = value;
                OnPropertyChanged("Port");
                OnPropertyChanged("Url");
            }
        }

        /// <summary>
        /// Gets the URL.
        /// </summary>
        /// <value>The URL.</value>
        [System.Xml.Serialization.XmlIgnore]
        public string Url
        {
            get
            {
                return "http:\\\\localhost:" + this.Port;
            }
        }
    }
}
