﻿namespace RedPoint.ReefStatus.Common.UI
{
    using System;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Threading;
    using System.Windows.Threading;

    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;

    public class ImportFromProfilux
    {
        /// <summary>
        /// Starts the specified callback.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <param name="controler">The controler.</param>
        /// <param name="dispatcher">The dispatcher.</param>
        public static void Start(IProgressCallback callback, Controler controler, Dispatcher dispatcher)
        {
            new Thread(() =>
                    {
                        if (callback != null)
                        {
                            try
                            {
                                callback.Begin(Language.GetResource("strImport"));

                                Collection<ItemDataRow> data = controler.Commands.GetDataPoints(callback);

                                if (callback.IsAborting)
                                {
                                    return;
                                }

                                using (var access = ReefStatusSettings.Instance.Logging.Connection.Create())
                                {
                                    access.AddLog(data, callback, controler.Id);
                                }

                                foreach (var item in controler.Probes.Cast<Probe>().Where(item => item.SaveToDatabase))
                                {
                                    var item1 = item;
                                    dispatcher.BeginInvoke(
                                        new Action(
                                            () =>
                                                {
                                                    if (item1.HasGraph)
                                                    {
                                                        item1.Graph.Refresh();
                                                    }

                                                    if (item1.HasDataPoints)
                                                    {
                                                        item1.DataPoints.Refresh();
                                                    }

                                                    if (item1.HasReport)
                                                    {
                                                        item1.Report.UpdateReport();
                                                    }
                                                }));
                                }
                            }
                            catch (ReefStatusException ex)
                            {
                                Logger.Instance.LogError(ex);
                            }
                            finally
                            {
                                callback.End();
                            }
                        }
                    }).Start();
        }
    }
}
