﻿// <copyright file="DataSettingsForm.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Windows.Forms;

    /// <summary>
    /// Data settings dialog
    /// </summary>
    public partial class DataSettingsForm : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DataSettingsForm"/> class.
        /// </summary>
        public DataSettingsForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets or sets the default units.
        /// </summary>
        /// <value>The default units.</value>
        public string DefaultUnits { get; set; }

        /// <summary>
        /// Gets or sets the default name.
        /// </summary>
        /// <value>The default name.</value>
        public string DefaultName { get; set; }

        /// <summary>
        /// Handles the Click event of the color control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Color_Click(object sender, EventArgs e)
        {
            ColorDialog dialog = new ColorDialog {Color = color.BackColor};
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color.BackColor = dialog.Color;
            }
        }

        /// <summary>
        /// Handles the Click event of the defaultName control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DefaultName_Click(object sender, EventArgs e)
        {
            displayName.Text = DefaultName;
        }

        /// <summary>
        /// Handles the Click event of the defaultUnits control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DefaultUnits_Click(object sender, EventArgs e)
        {
            units.Text = DefaultUnits;
        }
    }
}
