﻿// <copyright file="GraphForm.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Collections.ObjectModel;
    using System.Windows.Forms;
    using Common.ProfiLux;

    /// <summary>
    /// Graphing window
    /// </summary>
    public partial class GraphForm : Form
    {

        private bool forceClose;

        /// <summary>
        /// Initializes a new instance of the <see cref="GraphForm"/> class.
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="parent">The parent.</param>
        public GraphForm(BaseInfo settings, Form parent)
        {
            this.Settings = settings;
            InitializeComponent();
            panel1.Setup(settings, true);
            Text = settings.DisplayName;
            MdiParent = parent;
        }

        /// <summary>
        /// Gets the settings.
        /// </summary>
        /// <value>The settings.</value>
        public BaseInfo Settings { get; private set; }

        /// <summary>
        /// The control text
        /// </summary>
        /// <value>the text of the control</value>
        /// <returns>The text associated with this control.</returns>
        public override sealed string Text
        {
            get { return base.Text; }
            set { base.Text = value; }
        }

        /// <summary>
        /// Forces the close.
        /// </summary>
        public void ForceClose()
        {
            forceClose = true;
            Close();
        }

        /// <summary>
        /// Adds the point.
        /// </summary>
        /// <param name="date">The date.</param>
        /// <param name="value">The value.</param>
        public void AddPoint(DateTime date, double value)
        {
            panel1.AddPoint(date, value);
        }

        /// <summary>
        /// Adds the points to the graph.
        /// </summary>
        /// <param name="points">The points.</param>
        public void AddPoints(Collection<Common.ProfiLux.DataPoint> points)
        {
            panel1.AddPoints(points, true);
        }

        /// <summary>
        /// Handles the FormClosing event of the GraphForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void GraphForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!forceClose)
            {
                if (e.CloseReason != CloseReason.MdiFormClosing)
                {
                    Settings.Display = false;
                    ((IReefStatusFrame)MdiParent).RemoveGraph(Settings);
                }
            }
        }
    }
}