﻿// <copyright file="Remote.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Drawing;
    using System.Windows.Forms;

    using RedPoint.ReefStatus.Common.Core;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;

    /// <summary>
    /// The remote control window
    /// </summary>
    public partial class Remote : Form
    {
        private readonly System.Windows.Forms.Timer timerDisplayText = new System.Windows.Forms.Timer();

        /// <summary>
        /// Initializes a new instance of the <see cref="Remote"/> class.
        /// </summary>
        public Remote()
        {
            DataService.Instance.OnUpdateDisplayText += this.UpdateDisplayText;
            InitializeComponent();
            InitializeLanguageSettings();

            timerDisplayText.Interval = 1000;
            timerDisplayText.Tick += TimeDrisplayText_Tick;
        }

        #region Timers
        /// <summary>
        /// Handles the Tick event of the timeDrisplayText control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TimeDrisplayText_Tick(object sender, EventArgs e)
        {
            if (ReefStatusSettings.Instance.Controlers.Count != 0)
            {

            }

            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        #endregion

        public void InitializeLanguageSettings()
        {
            this.Text = "Remote";
            this.statusText.Text = "Not Connected";
            this.rightButton.Text = "Right";
            this.enterButton.Text = "Enter";
            this.downButton.Text = "Down";
            this.upButton.Text = "Up";
            this.leftButton.Text = "Left";
            this.escButton.Text = "Esc";
            this.viewText.Text = string.Empty;
        }

        #region RemoteControl

        /// <summary>
        /// Handles the Click event of the upButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void upButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Up);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Handles the Click event of the leftButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void leftButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Left);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Handles the Click event of the enterButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void enterButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Enter);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Handles the Click event of the rightButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void rightButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Right);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Handles the Click event of the escButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void escButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Esc);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Handles the Click event of the downButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void downButton_Click(object sender, EventArgs e)
        {
            ReefStatusSettings.Instance.FirstControler.Commands.SendKeyCommand(FaceplateKey.Down);
            ReefStatusSettings.Instance.FirstControler.Commands.UpdateDisplayText();
        }

        /// <summary>
        /// Updates the display text.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <param name="view">The view.</param>
        /// <param name="controler">The controler.</param>
        public void UpdateDisplayText(string status, string view, Image displayImage, Controler controler)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new Action(() => this.UpdateDisplayText(status, view, displayImage, controler)));
            }
            else
            {
                this.statusText.Text = status;
                this.viewText.Text = view;
            }
        }

        #endregion

        private void downButton_VisibleChanged(object sender, EventArgs e)
        {
            if(Visible)
            {
                timerDisplayText.Start();
            }
            else
            {
                timerDisplayText.Stop();
            }
        }
    }
}
