﻿// <copyright file="Report.cs" company="Redpoint Apps">
// Copyright (c) Redpoint Apps. All rights reserved.
// </copyright>

namespace RedPoint.ReefStatus
{
    using System;
    using System.Collections.ObjectModel;
    using System.Globalization;
    using System.Windows.Forms;

    using RedPoint.ReefStatus.Common.Database;
    using RedPoint.ReefStatus.Common.ProfiLux;
    using RedPoint.ReefStatus.Common.Settings;

    /// <summary>
    /// Report View
    /// </summary>
    public partial class Report : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Report"/> class.
        /// </summary>
        /// <param name="item">The item.</param>
        public Report(BaseInfo item)
        {
            InitializeComponent();
            InitializeLanguageSettings();
            this.Item = item;
            UpdateReport();
        }

        /// <summary>
        /// Gets the item.
        /// </summary>
        /// <value>The item.</value>
        public BaseInfo Item { get; private set; }

        /// <summary>
        /// Updates the report.
        /// </summary>
        public void UpdateReport()
        {
            reportList.Items.Clear();

            Collection<DataPoint> data;
            using (IDataAccess access = ReefStatusSettings.Instance.Logging.Connection.Create())
            {
                data = new Collection<DataPoint>(access.GetDataPoints(Item.Id, true, this.Item.Controler.Id));
            }

            Stats stats = MemoryDataAccess.GetStats(DateTime.Now, DateTime.Now.AddDays(-1), data, true);
            stats.ApplyConverter(this.Item);
            ListViewItem item = reportList.Items.Add("Day");
            item.SubItems.Add(Math.Round(stats.Max, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Min, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Average, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.StdDeviation, 2).ToString(CultureInfo.CurrentCulture));

            stats = MemoryDataAccess.GetStats(DateTime.Now, DateTime.Now.AddDays(-7), data, true);
            stats.ApplyConverter(this.Item);
            item = reportList.Items.Add("Week");
            item.SubItems.Add(Math.Round(stats.Max, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Min, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Average, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.StdDeviation, 2).ToString(CultureInfo.CurrentCulture));

            stats = MemoryDataAccess.GetStats(DateTime.Now, DateTime.Now.AddMonths(-1), data, true);
            stats.ApplyConverter(this.Item);
            item = reportList.Items.Add("Month");
            item.SubItems.Add(Math.Round(stats.Max, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Min, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Average, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.StdDeviation, 2).ToString(CultureInfo.CurrentCulture));

            stats = MemoryDataAccess.GetStats(DateTime.Now, DateTime.Now.AddYears(-1), data, true);
            stats.ApplyConverter(this.Item);
            item = reportList.Items.Add("Year");
            item.SubItems.Add(Math.Round(stats.Max, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Min, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Average, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.StdDeviation, 2).ToString(CultureInfo.CurrentCulture));

            stats = MemoryDataAccess.GetStats(DateTime.Now, DateTime.Now.AddYears(-100), data, true);
            stats.ApplyConverter(this.Item);
            item = reportList.Items.Add("All");
            item.SubItems.Add(Math.Round(stats.Max, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Min, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.Average, 2).ToString(CultureInfo.CurrentCulture));
            item.SubItems.Add(Math.Round(stats.StdDeviation, 2).ToString(CultureInfo.CurrentCulture));

        }

        /// <summary>
        /// Initializes the language settings.
        /// </summary>
        public void InitializeLanguageSettings()
        {
            Text = "Report";
            columnHeader1.Text = "Range";
            columnHeader2.Text = "Max";
            columnHeader3.Text = "Min";
            columnHeader4.Text = "Average";
            columnHeader5.Text = "Standard Deviation";
        }
    }
}
